<?php

namespace Sia\Vpos\Controller\Payment;

/**
 * Controller per la pagina di risultato pagamento mediante VposPay
 *
 * @author Cooder S.r.l. <info@cooder.it>
 */
class Done extends \Magento\Framework\App\Action\Action
{
    
    /**
     * Checkout Session
     * 
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;
    
    /**
     * Logger del modulo
     * 
     * @var \Sia\Vpos\Logger\Logger
     */
    protected $_baseLogger;
    
    /**
     * Scope Config
     *
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;
    
    /**
     * Transaction Factory
     *
     * @var \Sia\Vpos\Model\TransactionFactory
     */
    protected $_transactionFactory;
    
    /**
     * Store Manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    
    /**
     * Costruttore
     * 
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Sia\Vpos\Logger\Logger $baseLogger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Sia\Vpos\Model\TransactionFactory $transactionFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Sia\Vpos\Logger\Logger $baseLogger,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Sia\Vpos\Model\TransactionFactory $transactionFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) 
    {
        parent::__construct($context);
        $this->_checkoutSession = $checkoutSession;
        $this->_baseLogger = $baseLogger;
        $this->_scopeConfig = $scopeConfig;
        $this->_transactionFactory = $transactionFactory;
        $this->_storeManager = $storeManager;
    }
    
    /**
     * Esecuzione delle verifiche sulla risposta di 
     * pagamento dal server Sia
     */
    public function execute()
    {
        $this->_baseLogger->debug('Verify result: start');
        
        // Recupero ordine in sessione
        $lastOrder = $this->_checkoutSession->getLastRealOrder();
        
        // Istanzio metodo di pagamento
        $payment = $lastOrder->getPayment();
        if (!$payment instanceof \Magento\Payment\Model\InfoInterface) {
            $this->_baseLogger->debug('Payment object is not a PaymentInfo!');
            return $this->_sendErrorResponse();
        }
        
        $paymentMethodInstance = $payment->getMethodInstance();
        if (!$paymentMethodInstance instanceof \Sia\Vpos\Model\VposPay) {
            $this->_baseLogger->debug(
                'Notification is not about an VposPay payment! Payment method object: '.
                get_class($paymentMethodInstance)
            );
            return $this->_sendErrorResponse();
        }
        
        // Recupero parametri in sessione
        $params = $this->getRequest()->getParams();
                
        // Eseguo verifica dei dati di pagamento
        $result = $paymentMethodInstance->paymentVerify($params);
        
        // Eseguo redirect a seconda del risultato
        if ($result) {
            $this->_baseLogger->debug('Verify result: success');
            $url = $this->_scopeConfig->getValue('payment/vpos/url_success', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE, $this->_storeManager->getWebsite()->getId());
            if(empty($url)) {
                return $this->_redirect('checkout/onepage/success');
            } else {
                return $this->_redirect($url);
            }
        }
        
        $this->_baseLogger->debug('Verify result: error');
        $url = $this->_scopeConfig->getValue('payment/vpos/url_failure', \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE, $this->_storeManager->getWebsite()->getId());
        if(empty($url)) {
            return $this->_redirect('checkout/onepage/failure');
        } else {
            return $this->_redirect($url);
        }
    }
    
    /**
     * Invia response di errore
     * 
     * @return \Magento\Framework\App\ResponseInterface
     */
    protected function _sendErrorResponse()
    {
        $response = $this->getResponse();
        $response->setHttpResponseCode(500);
        return $response;
    }
}