<?php

namespace Sia\Vpos\Model\Export;

/**
 * Override per classe \Magento\Ui\Model\Export\MetadataProvider
 * 
 * @author Cooder S.r.l. <info@cooder>
 */
class MetadataProvider extends \Magento\Ui\Model\Export\MetadataProvider
{
    
    /**
     * Source per Authorization Mode
     * 
     * @var \Sia\Vpos\Model\Config\Source\Authorization\Mode
     */
    protected $_sourceAuthorizationMode;
    
    /**
     * Source per Authorization Network
     *
     * @var \Sia\Vpos\Model\Config\Source\Authorization\Network
     */
    protected $_sourceAuthorizationNetwork;
    
    /**
     * Source per Authorization Payment Type
     *
     * @var \Sia\Vpos\Model\Config\Source\Authorization\PaymentType
     */
    protected $_sourceAuthorizationPaymentType;
    
    /**
     * Source per Authorization Transaction Result
     *
     * @var \Sia\Vpos\Model\Config\Source\Authorization\TransactionResult
     */
    protected $_sourceAuthorizationTransactionResult;
    
    /**
     * Source per Authorization Transaction Status
     *
     * @var \Sia\Vpos\Model\Config\Source\Authorization\TransactionStatus
     */
    protected $_sourceAuthorizationTransactionStatus;

    /**
     * Source per Transaction Result
     *
     * @var \Sia\Vpos\Model\Config\Source\Transaction\Result
     */
    protected $_sourceTransactionResult;
    
    /**
     * Source per Transaction Source Type
     *
     * @var \Sia\Vpos\Model\Config\Source\Transaction\SrcType
     */
    protected $_sourceTransactionSrcType;
    
    /**
     * Source per Transaction Status
     *
     * @var \Sia\Vpos\Model\Config\Source\Transaction\Status
     */
    protected $_sourceTransactionStatus;
    
    /**
     * Costurttore
     * 
     * @param \Sia\Vpos\Model\Config\Source\Authorization\Mode $sourceAuthorizationMode
     * @param \Sia\Vpos\Model\Config\Source\Authorization\Network $sourceAuthorizationNetwork
     * @param \Sia\Vpos\Model\Config\Source\Authorization\PaymentType $sourceAuthorizationPaymentType
     * @param \Sia\Vpos\Model\Config\Source\Authorization\TransactionStatus $sourceAuthorizationTransactionStatus
     * @param \Sia\Vpos\Model\Config\Source\Authorization\TransactionResult $sourceAuthorizationTransactionResult
     * @param \Sia\Vpos\Model\Config\Source\Transaction\Result $sourceTransactionResult
     * @param \Sia\Vpos\Model\Config\Source\Transaction\SrcType $sourceTransactionSrcType
     * @param \Sia\Vpos\Model\Config\Source\Transaction\Status $sourceTransactionStatus
     * @param \Magento\Ui\Component\MassAction\Filter $filter
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate
     * @param \Magento\Framework\Locale\ResolverInterface $localeResolver
     * @param string $dateFormat
     * @param array $data
     */
    public function __construct(
        \Sia\Vpos\Model\Config\Source\Authorization\Mode $sourceAuthorizationMode,
        \Sia\Vpos\Model\Config\Source\Authorization\Network $sourceAuthorizationNetwork,
        \Sia\Vpos\Model\Config\Source\Authorization\PaymentType $sourceAuthorizationPaymentType,
        \Sia\Vpos\Model\Config\Source\Authorization\TransactionStatus $sourceAuthorizationTransactionStatus,
        \Sia\Vpos\Model\Config\Source\Authorization\TransactionResult $sourceAuthorizationTransactionResult,
        \Sia\Vpos\Model\Config\Source\Transaction\Result $sourceTransactionResult,
        \Sia\Vpos\Model\Config\Source\Transaction\SrcType $sourceTransactionSrcType,
        \Sia\Vpos\Model\Config\Source\Transaction\Status $sourceTransactionStatus,
        \Magento\Ui\Component\MassAction\Filter $filter,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Locale\ResolverInterface $localeResolver,
        $dateFormat = 'M j, Y H:i:s A',
        array $data = []
    )
    {
        parent::__construct($filter, $localeDate, $localeResolver, $dateFormat, $data);    
        $this->_sourceAuthorizationMode = $sourceAuthorizationMode;
        $this->_sourceAuthorizationNetwork = $sourceAuthorizationNetwork;
        $this->_sourceAuthorizationPaymentType = $sourceAuthorizationPaymentType;
        $this->_sourceAuthorizationTransactionResult = $sourceAuthorizationTransactionResult;
        $this->_sourceAuthorizationTransactionStatus = $sourceAuthorizationTransactionStatus;
        $this->_sourceTransactionResult = $sourceTransactionResult;
        $this->_sourceTransactionSrcType = $sourceTransactionSrcType;
        $this->_sourceTransactionStatus = $sourceTransactionStatus;
    }
    
    /**
     * {@inheritDoc}
     * @see \Magento\Ui\Model\Export\MetadataProvider::getOptions()
     */
    public function getOptions(): array
    {
        $options = parent::getOptions();
        
        $component = $this->filter->getComponent();
        if($component->getName() == 'vpos_transactions_listing') {
            $options['auth_accounting_mode'] = $this->_sourceAuthorizationMode->toArray();  
            $options['auth_author_mode'] = $this->_sourceAuthorizationMode->toArray();  
            $options['auth_network'] = $this->_sourceAuthorizationNetwork->toArray();  
            $options['auth_payment_type'] = $this->_sourceAuthorizationPaymentType->toArray();  
            $options['auth_transaction_result'] = $this->_sourceAuthorizationTransactionResult->toArray();  
            $options['auth_transaction_status'] = $this->_sourceAuthorizationTransactionStatus->toArray();  
            $options['result'] = $this->_sourceTransactionResult->toArray();  
            $options['src_type'] = $this->_sourceTransactionSrcType->toArray();  
            $options['status'] = $this->_sourceTransactionStatus->toArray();  
        }
        
        return $options;
    }
    
}